from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class nb_enf(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Nombre d'enfants"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        """
        TODO: fixme
        """
        age = foyer_fiscal.members("age", period=period.first_month)
        famille = parameters(period.start).impot_revenu.deductions.famille
        # res =+ (
        #    (ag < 20) +
        #    (ag < 25)*not_(boursier)*()
        #    )
        condition = (age >= 0) * (age <= famille.age)
        return foyer_fiscal.sum(condition, role=FoyerFiscal.PERSONNE_A_CHARGE)


class nb_enf_sup(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Nombre d'enfants étudiants du supérieur non boursiers"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        """
        TODO: Nombre d'enfants étudiants du supérieur non boursiers
        """
        # age = foyer_fiscal.members('age', period = period)
        etudiant = foyer_fiscal.members("etudiant", period=period)
        boursier = foyer_fiscal.members("boursier", period=period)

        return foyer_fiscal.sum(etudiant * not_(boursier))


class nb_infirme(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Nombre d'enfants infirmes"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        """
        Nombre d'enfants infirmes
        """
        infirme = foyer_fiscal.members("handicap", period=period) >= 3

        return foyer_fiscal.sum(1 * infirme, role=FoyerFiscal.PERSONNE_A_CHARGE)


class nb_parents(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Nombre de parents"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        """
        Nombre de parents
        """
        return (foyer_fiscal.declarant_principal("age", period) > 20) + (
            foyer_fiscal.conjoint("age", period) > 20
        )


class chef_de_famille(Variable):
    value_type = bool
    entity = FoyerFiscal
    label = "Indicatrice de chef de famille"
    definition_period = YEAR
    set_input = set_input_dispatch_by_period

    # Du point de vue fiscal, est considéré chef de famille :
    # - L’époux ;
    # - Le divorcé ou la divorcée qui a la garde des enfants (divorce & enfnats);
    # - Le veuf ou la veuve même sans enfants à charge;
    # - L’adoptant ou l’adoptante (adoptant). TODO
    # Cependant, l’épouse a la qualité de chef de famille dans les deux cas suivants :
    # - Lorsqu’elle justifie que le mari ne dispose d’aucune source de revenu durant l’année de réalisation du
    #   revenu. Tel est le cas d’une femme qui dispose d’un revenu et dont le mari, poursuivant des études, ne dispose
    #   d’aucun revenu propre. (marie & sexe & revenu_epoux == 0 & revenu_individu > 0) TODO
    # - Lorsque remariée, elle a la garde d’enfants issus d’un précédent mariage. TODO
    # Compte tenu de ce qui précède, n’est pas considéré comme chef de famille et ne bénéficie d’aucune déduction :
    # - Le célibataire ou la célibataire ;
    # - Le divorcé ou la divorcée qui n’a pas la garde des enfants ;
    # - La femme durant le mariage (sauf si elle dispose d’un revenu alors que son mari ne dispose d’aucun revenu) ;
    # - L’époux qui ne dispose pas d’une source de revenu. Dans ce cas, l’épouse acquiert la qualité de chef de famille
    #   au cas où elle réalise des revenus.

    def formula(foyer_fiscal, period):
        male = foyer_fiscal.declarant_principal("male", period=period)
        marie = foyer_fiscal.declarant_principal("marie", period=period)
        divorce = foyer_fiscal.declarant_principal("divorce", period=period)
        veuf = foyer_fiscal.declarant_principal("veuf", period=period)
        nb_enf = foyer_fiscal("nb_enf", period=period)
        chef_de_famille = (
            veuf | (marie & male) | (divorce & (nb_enf > 0))  # | (marie & (not male)) |
        )
        return chef_de_famille


# Revenus catégoriels (voir répertoire idoine)
# 1. Bénéfices industriels et commerciaux  bic.py
# 2. Bénéfices des professions non commerciales  bnc.py
# 3. Bénéfices de l'exploitation agricole et de pêche beap.py
# 4. Revenus fonciers foncier.py
# 5. Traitements, salaires, indemnités, pensions et rentes viagères  tspr.py
# 6. Revenus de valeurs mobilières et de capitaux mobiliers  rvcm.py
# 7. revenus de source étrangère

# Déroulé du calcul de l'irpp


class rng(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Revenu net global"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        return (
            foyer_fiscal("bic", period=period)
            + foyer_fiscal("bnc", period=period)
            + foyer_fiscal("beap", period=period)
            + foyer_fiscal("revenus_fonciers", period=period)
            + foyer_fiscal("tspr", period=period)
            + foyer_fiscal("rvcm", period=period)
            + foyer_fiscal("revenus_source_etrangere", period=period)
        )


#############################
#    Déductions
#############################

# # 1/ Au titre des revenus et bénéfices provenant de l’activité

# # 2/ Autres déductions


class deduction_interets(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Déductions intérêts issus de comptes spéciaux ou d'obligations"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        compte_special_epargne_banque = foyer_fiscal("compte_special_epargne_banque")
        compte_special_epargne_cent = foyer_fiscal("compte_special_epargne_cent")
        emprunt_obligataire = foyer_fiscal("emprunt_obligataire")
        deductions = parameters(period).impot_revenu.deductions
        return max_(
            max_(
                (
                    max_(compte_special_epargne_banque, deductions.banq.plaf)
                    + max_(compte_special_epargne_cent, deductions.cent.plaf)
                ),
                deductions.banq.plaf,
            )
            + max_(emprunt_obligataire, deductions.oblig.plaf),
            deductions.oblig.plaf,
        )


class deduction_famille(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Déductions pour situation et charges de famille"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        # rng = foyer_fiscal('rng', period = period)
        chef_de_famille = foyer_fiscal("chef_de_famille", period=period)
        nb_enf = foyer_fiscal("nb_enf", period=period)
        nb_infirme = foyer_fiscal("nb_infirme", period=period)
        famille = parameters(period.start).impot_revenu.deductions.famille
        #  chef de famille
        chef_de_famille = famille.chef_de_famille * chef_de_famille

        infirme = famille.infirme * nb_infirme
        nb_enf = max_(nb_enf - nb_infirme, 0)
        enf = (
            (nb_enf >= 1) * famille.enf1
            + (nb_enf >= 2) * famille.enf2
            + (nb_enf >= 3) * famille.enf3
            + (nb_enf >= 4) * famille.enf4
        )

        #    sup = P.enf_sup * nb_enf_sup

        #    parent = min_(P.parent_taux * rng, P.parent_max)
        #    return chef_de_famille + enf + sup + infirme + parent

        res = chef_de_famille + enf + infirme
        return res


class deduction_rente(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Arrérages et rentes payées à titre obligatoire et gratuit"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        rente = foyer_fiscal("rente", period=period)

        return rente  # TODO:


class deduction_assurance_vie(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Primes afférentes aux contrats d'assurance-vie collectifs ou individuels"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        primes_assurance_vie = foyer_fiscal.members(
            "prime_assurance_vie", period=period
        )
        somme_primes_assurance_vie = foyer_fiscal.sum(primes_assurance_vie)
        marie = foyer_fiscal.declarant_principal("statut_marital", period=period)
        nb_enf = foyer_fiscal("nb_enf", period=period)
        assurance_vie = parameters(period.start).impot_revenu.deductions.assurance_vie
        deduction = min_(
            somme_primes_assurance_vie,
            assurance_vie.plaf
            + marie * assurance_vie.conj_plaf
            + nb_enf * assurance_vie.enf_plaf,
        )
        return deduction


#     - Les remboursements des prêts universitaires en principal et en intérêts
#    - Revenus et bénéfices réinvestis dans les conditions et limites
#    prévues par la législation en vigueur dont notamment :
#     les revenus provenant de l'exportation, totalement pendant 10
#    ans à partir de la première opération d'exportation;
#     les revenus provenant de l’hébergement et de la restauration
#    des étudiants pendant dix ans ;
#     les revenus provenant du courtage international dans la limite
#    de 50% pendant 10 ans à partir de la première opération de courtage
#    international;
#     les montants déposés dans les comptes-épargne pour
#    l’investissement et dans les comptes-épargne en actions dans la
#    limite de 20.000D par an et sous réserve du minimum d’IR ;
#     la plus-value provenant des opérations de transmission des
#    entreprises en difficultés économiques ou des entreprises sous
#    forme de participations ou d’actifs pour départ du propriétaire à la
#    retraite ou pour incapacité de poursuivre la gestion et ce, sous
#    certaines conditions,
#     la plus-value provenant de l’apport d’une entreprise individuelle
#    au capital d’une société ;
#     les revenus réinvestis dans la souscription au capital des
#    entreprises dans les conditions prévues par la législation relative aux
#    incitations fiscales;
#     les revenus réinvestis dans la réalisation de projets
#    d’hébergement et de restauration universitaires sous réserve du
#    minimum d’IR.
#     Les revenus provenant de la location des terres agricoles
#    réservées aux grandes cultures objet de contrat de location conclus
#    pour une période minimale de trois ans.


class deduction_smig(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Déduction supplémentaire pour les salariés payés au SMIG et SMAG"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        chef = foyer_fiscal("chef_de_famille", period=period)

        return 0 * chef  # TODO: voir avec tspr


class deductions(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Revenu net imposable"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        return (
            foyer_fiscal("deduction_famille", period=period)
            + foyer_fiscal.declarant_principal("rente", period=period)
            + foyer_fiscal("deduction_assurance_vie", period=period)
            + foyer_fiscal.sum(
                foyer_fiscal.members(
                    "plus_value_cession_actifs_cotes_bourse", period=period
                )
            )
            + foyer_fiscal.sum(
                foyer_fiscal.members("interets_acquisition_logement", period=period)
            )
        )


class revenu_net_imposable(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Revenu net imposable"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        """
        Revenu net imposable ie soumis à au barême de l'impôt après déduction des dépenses
        et charges professionnelles et des revenus non soumis à l'impôt
        """
        return max_(foyer_fiscal("rng", period) - foyer_fiscal("deductions", period), 0)


class impot_revenu_brut(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Impôt brut avant non-imposabilité"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        revenu_net_imposable = foyer_fiscal("revenu_net_imposable", period=period)
        bareme = parameters(period.start).impot_revenu.bareme
        impot_revenu_brut = bareme.calc(revenu_net_imposable)
        return impot_revenu_brut


class exoneration(Variable):
    value_type = bool
    entity = FoyerFiscal
    label = "Exoneration de l'impôt sur le revenu des personnes physiques"
    definition_period = YEAR
    end = "2016-12-31"

    def formula_2014(foyer_fiscal, period, parameters):
        # Les éligibles ne doivent percevoir que des salaires et des pensions
        rng = foyer_fiscal("rng", period=period)
        tspr = foyer_fiscal("tspr", period=period)
        eligble = rng == tspr
        # Condition de revenu
        deduction_famille = foyer_fiscal("deduction_famille", period=period)
        condition_de_revenu = (rng - deduction_famille) <= parameters(
            period.start
        ).impot_revenu.exoneration.seuil
        return eligble * condition_de_revenu


class irpp(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Impôt sur le revenu des personnes physiques"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        impot_revenu_brut = foyer_fiscal("impot_revenu_brut", period=period)
        exoneration = foyer_fiscal("exoneration", period=period)
        return impot_revenu_brut * not_(exoneration)


class irpp_salarie_preleve_a_la_source(Variable):
    value_type = float
    entity = Individu
    label = "Impôt sur le revenu des personnes physiques prélevé à la source pour les salariés"
    definition_period = MONTH

    def formula(individu, period, parameters):
        salaire_imposable = individu("salaire_imposable", period=period)
        deduction_famille_annuelle = individu.foyer_fiscal(
            "deduction_famille", period=period.this_year
        )

        return calcule_impot_revenu_brut(
            salaire_imposable,
            deduction_famille_annuelle,
            period,
            parameters,
        )


class irpp_net_a_payer(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "IRPP net des prélèvements à la source"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        return foyer_fiscal("irpp", period) - foyer_fiscal.sum(
            foyer_fiscal.members(
                "irpp_salarie_preleve_a_la_source", period, options=[ADD]
            )
        )


# Utils


def calcule_base_imposable(
    salaire_mensuel, deduction_famille_annuelle, period, parameters
):
    revenu_assimile_salaire = salaire_mensuel
    smig_40h_mensuel = parameters(period.start).marche_travail.smig_40h_mensuel
    smig = revenu_assimile_salaire <= smig_40h_mensuel
    tspr = parameters(period.start).impot_revenu.tspr
    abattement_frais_professionnels = min_(
        tspr.abat_sal * revenu_assimile_salaire, tspr.max_abat_sal / 12
    )

    if period.start.year >= 2011:
        revenu_assimile_salaire_apres_abattement = max_(
            revenu_assimile_salaire
            - abattement_frais_professionnels
            - max_(
                smig * tspr.abattement_pour_salaire_minimum,
                (revenu_assimile_salaire <= tspr.smig_ext)
                * tspr.abattement_pour_salaire_minimum,
            ),
            0,
        )
    else:
        revenu_assimile_salaire_apres_abattement = max_(
            revenu_assimile_salaire
            - abattement_frais_professionnels
            - smig * tspr.abattement_pour_salaire_minimum,
            0,
        )

    non_exonere = revenu_assimile_salaire_apres_abattement >= 0
    if 2014 <= period.start.year <= 2016:
        non_exonere = (
            12 * revenu_assimile_salaire_apres_abattement - deduction_famille_annuelle
        ) > parameters(period.start).impot_revenu.exoneration.seuil

    return non_exonere, revenu_assimile_salaire_apres_abattement


def calcule_impot_revenu_brut(
    salaire_mensuel, deduction_famille_annuelle, period, parameters
):
    non_exonere, revenu_assimile_salaire_apres_abattement = calcule_base_imposable(
        salaire_mensuel, deduction_famille_annuelle, period, parameters
    )
    bareme = parameters(period.start).impot_revenu.bareme
    return (
        non_exonere
        * bareme.calc(
            12 * revenu_assimile_salaire_apres_abattement - deduction_famille_annuelle
        )
    ) / 12



"""7. revenus de source étrangère"""

from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class revenus_source_etrangere(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Autres revenus (revenus de source étrangère n’ayant pas subi l’impôt dans le pays d'origine)"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        salaire_etranger = foyer_fiscal.declarant_principal(
            "salaire_etranger", period=period
        )
        pension_etranger_transferee = foyer_fiscal.declarant_principal(
            "pension_etranger_transferee", period=period
        )
        pension_etranger_non_transferee = foyer_fiscal.declarant_principal(
            "pension_etranger_non_transferee", period=period
        )
        autres_revenus_etranger = foyer_fiscal.declarant_principal(
            "autres_revenus_etranger", period=period
        )
        tspr = parameters(period.start).impot_revenu.tspr
        abattement_frais_professionnels = min_(
            salaire_etranger * tspr.abat_sal, tspr.max_abat_sal
        )

        return (
            salaire_etranger
            - abattement_frais_professionnels
            + pension_etranger_non_transferee * (1 - tspr.abat_pen)
            + pension_etranger_transferee * (1 - tspr.abat_pen_etr)
            + autres_revenus_etranger
        )



from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class TypesRegimeSecuriteSocialeCotisant(Enum):
    __order__ = (
        "neant rsna rsa rsaa rtns rtte re rtfr raci salarie_cnrps pensionne_cnrps"
    )
    # Needed to preserve the enum order in Python 2
    neant = "Néant"
    rsna = "Régime des salariés non agricoles"
    rsa = "Régime des salariés agricoles"
    rsaa = "Régime des salariés agricoles amélioré"
    rtns = "Régime des travailleurs non salariés (secteurs agricole et non agricole)"
    rtte = "Régime des travailleurs tunisiens à l'étranger"
    re = "Régime des étudiants, diplômés de l'enseignement supérieur et stagiaires"
    rtfr = "Régime des travailleurs à faibles revenus (gens de maisons, travailleurs de chantiers, et artisans travaillant à la pièce)"
    raci = "Régime des artistes, créateurs et intellectuels"
    salarie_cnrps = "Régime des salariés affilés à la Caisse nationale de retraite et de prévoyance sociale (CNRPS)"
    pensionne_cnrps = "Régime des pensionnés affilés à la Caisse nationale de retraite et de prévoyance sociale (CNRPS)"
    # references :
    # http://www.social.gov.tn/index.php?id=49&L=0
    # http://www.paie-tunisie.com/412/fr/83/reglementations/regimes-de-securite-sociale.aspx


class TypesRegimeSecuriteSocialeRetraite(Enum):
    __order__ = "neant rsna rtte raci rtns_na rtns_a rtc rsa rsaa cnrps"
    # Needed to preserve the enum order in Python 2
    neant = "Néant"
    rsna = "Régime des salariés non agricoles"
    rtte = "Régime des travailleurs tunisiens à l'étranger"
    raci = "Régime des artistes, créateurs etiIntellectuels"
    rtns_na = "Régime des travailleurs non salariés des secteurs non agricoles"
    rtns_a = "Régime des travailleurs non salariés des secteurs agricoles"
    rtc = "Régime des travailleurs de chantiers"
    rsa = "Régime des salariés agricoles"
    rsaa = "Régime des salariés agricoles amélioré"
    cnrps = "Régime des pensionnés affilés à la Caisse Nationale de Retraite et de Prévoyance Sociale"
    # references :
    # http://www.social.gov.tn/index.php?id=49&L=0
    # http://www.paie-tunisie.com/412/fr/83/reglementations/regimes-de-securite-sociale.aspx


def compute_cotisation(
    individu, period, cotisation_type=None, bareme_name=None, parameters=None
):
    assert cotisation_type in ["employeur", "salarie"]

    assiette_cotisations_sociales = individu("assiette_cotisations_sociales", period)
    regime_securite_sociale_cotisant = individu(
        "regime_securite_sociale_cotisant", period
    )
    cotisations_sociales = parameters(
        period.start
    ).prelevements_sociaux.cotisations_sociales
    cotisation = individu.empty_array()
    types_regime_securite_sociale_cotisant = (
        regime_securite_sociale_cotisant.possible_values
    )

    regimes_prive = [
        "rsna",
        "rtte",
        "raci",
        "rtns",
        "rsa",
        "rsaa",
        "re",
        "rtfr",
    ]

    for regime in types_regime_securite_sociale_cotisant:
        if regime.name == "neant":
            continue

        if regime.name in regimes_prive:
            baremes_by_regime = cotisations_sociales.secteur_prive
        else:
            baremes_by_regime = cotisations_sociales.secteur_public

        if (
            f"cotisations_{cotisation_type}"
            not in baremes_by_regime[regime.name]._children
        ):
            continue
        baremes_by_name = getattr(
            baremes_by_regime[regime.name],
            f"cotisations_{cotisation_type}",
        )

        if bareme_name in ["maladie", "maternite", "deces"]:
            if "assurances_sociales" in baremes_by_name._children:
                baremes_assurances_sociales = getattr(
                    baremes_by_name, "assurances_sociales"
                )
                bareme = getattr(baremes_assurances_sociales, bareme_name)

            else:
                if bareme_name not in baremes_by_name._children:
                    continue
                bareme = getattr(baremes_by_name, bareme_name)

        else:
            if bareme_name not in baremes_by_name._children:
                continue
            bareme = getattr(baremes_by_name, bareme_name)

        if bareme is not None:
            if regime.name == "rtfr":
                smic = parameters(period.start).marche_travail.smig_48h_mensuel
                bareme = bareme.multiply_thresholds(smic)

            cotisation += bareme.calc(
                assiette_cotisations_sociales
                * (regime_securite_sociale_cotisant == regime),
            )

    return cotisation


class assiette_cotisations_sociales(Variable):
    value_type = float
    entity = Individu
    label = "Assiette des cotisations sociales"
    definition_period = MONTH

    def formula(individu, period):
        return individu("salaire_de_base", period) + individu("primes", period)


class regime_securite_sociale_cotisant(Variable):
    value_type = Enum
    possible_values = TypesRegimeSecuriteSocialeCotisant
    default_value = TypesRegimeSecuriteSocialeCotisant.neant
    entity = Individu
    label = "Régime de sécurité sociale du salarié"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class regime_securite_sociale_retraite(Variable):
    value_type = Enum
    possible_values = TypesRegimeSecuriteSocialeRetraite
    default_value = TypesRegimeSecuriteSocialeRetraite.neant
    entity = Individu
    label = "Régime de sécurité sociale du salarié"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class cotisations_sociales(Variable):
    value_type = float
    entity = Individu
    label = "Cotisations sociales"
    definition_period = MONTH

    def formula(individu, period):
        return individu("cotisations_employeur", period) + individu(
            "cotisations_salarie", period
        )


class cotisations_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation sociales employeur"
    definition_period = MONTH

    def formula(individu, period):
        cotisations_employeur = [
            "accident_du_travail_employeur",
            "deces_employeur",
            "fonds_special_etat",
            "famille_employeur",
            "maladie_employeur",
            "maternite_employeur",
            "protection_sociale_travailleurs_employeur",
            "retraite_employeur",
        ]
        return sum(
            individu(f"{cotisation}", period) for cotisation in cotisations_employeur
        )


class cotisations_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation sociales salarié"
    definition_period = MONTH

    def formula(individu, period):
        cotisations_salarie = [
            "accident_du_travail_salarie",
            "deces_salarie",
            "famille_salarie",
            "maladie_salarie",
            "maternite_salarie",
            "protection_sociale_travailleurs_salarie",
            "retraite_salarie",
            "soin_salarie",
            "ugtt",
        ]
        return sum(
            individu(f"{cotisation}", period) for cotisation in cotisations_salarie
        )


class accident_du_travail_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation accidents du travail et maladies professionnelles (employeur)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="employeur",
            bareme_name="accident_du_travail",
            parameters=parameters,
        )


class accident_du_travail_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation accidents du travail et maladies professionnelles (salarié)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="salarie",
            bareme_name="accident_du_travail",
            parameters=parameters,
        )


class deces_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation assurances sociales: décès (employeur)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="employeur",
            bareme_name="deces",
            parameters=parameters,
        )


class deces_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation assurances sociales: décès (salarié)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="salarie",
            bareme_name="deces",
            parameters=parameters,
        )


class famille_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation sociale allocations familiales (employeur)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="employeur",
            bareme_name="famille",
            parameters=parameters,
        )


class famille_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation sociale allocations familiales (salarié)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="salarie",
            bareme_name="famille",
            parameters=parameters,
        )


class fonds_special_etat(Variable):
    value_type = float
    entity = Individu
    label = "Fonds spécial de l'Etat"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="employeur",
            bareme_name="fonds_special_etat",
            parameters=parameters,
        )


class maladie_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation assurances sociales: maladie (employeur)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="employeur",
            bareme_name="maladie",
            parameters=parameters,
        )


class maladie_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation assurances sociales: maladie (salarié)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="salarie",
            bareme_name="maladie",
            parameters=parameters,
        )


class maternite_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation assurances sociales : maternité (employeur)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="employeur",
            bareme_name="maternite",
            parameters=parameters,
        )


class maternite_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation assurances sociales : maternité (salarié)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="salarie",
            bareme_name="maternite",
            parameters=parameters,
        )


class protection_sociale_travailleurs_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation protection sociale des travailleurs (employeur)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="employeur",
            bareme_name="protection_sociale_travailleurs",
            parameters=parameters,
        )


class protection_sociale_travailleurs_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation protection sociale des travailleurs (salarié)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="salarie",
            bareme_name="protection_sociale_travailleurs",
            parameters=parameters,
        )


class retraite_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation pensions de retraite (employeur)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="employeur",
            bareme_name="retraite",
            parameters=parameters,
        )


class retraite_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation pensions de retraite (salarié)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="salarie",
            bareme_name="retraite",
            parameters=parameters,
        )


class soin_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation soin (employeur)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="employeur",
            bareme_name="soin",
            parameters=parameters,
        )


class soin_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation soin (salarié)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type="salarie",
            bareme_name="soin",
            parameters=parameters,
        )


class salaire_imposable(Variable):
    value_type = float
    entity = Individu
    label = "Salaire imposable"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        return individu("assiette_cotisations_sociales", period) - individu(
            "cotisations_salarie", period
        )


class salaire_net_a_payer(Variable):
    value_type = float
    entity = Individu
    label = "Salaire net à payer (fiche de paie)"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2018_01_01(individu, period):
        return (
            individu("salaire_imposable", period)
            - individu("irpp_salarie_preleve_a_la_source", period)
            - individu("contribution_sociale_solidarite_prelevee_a_la_source", period)
        )

    def formula(individu, period):
        return individu("salaire_imposable", period) - individu(
            "irpp_salarie_preleve_a_la_source", period
        )


class salaire_super_brut(Variable):
    value_type = float
    entity = Individu
    label = "Salaires super bruts"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        return (
            individu("salaire_de_base", period=period)
            + individu("primes", period=period)
            + individu(
                "cotisations_employeur", period=period
            )  # Cotisations employeur sont négatives
        )


class ugtt(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation syndicale UGTT"
    definition_period = MONTH

    def formula(individu, period):
        # TODO put this value (3) in parameters
        return 3 * (
            individu("regime_securite_sociale_cotisant", period)
            == TypesRegimeSecuriteSocialeCotisant.salarie_cnrps
        )



from numpy import datetime64, timedelta64


from openfisca_tunisia.variables.base import *


class age(Variable):
    unit = "years"
    value_type = int
    default_value = AGE_INT_MINIMUM
    entity = Individu
    label = "Âge (en années) au premier jour du mois"
    definition_period = MONTH
    is_period_size_independent = True
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        has_birth = individu.get_holder("date_naissance").get_known_periods()
        if not has_birth:
            has_age_en_mois = bool(
                individu.get_holder("age_en_mois").get_known_periods()
            )
            if has_age_en_mois:
                return individu("age_en_mois", period) // 12

            # If age is known at the same day of another year, compute the new age from it.
            holder = individu.get_holder("age")
            start = period.start
            known_periods = holder.get_known_periods()
            if known_periods:
                for last_period in sorted(known_periods, reverse=True):
                    last_start = last_period.start
                    if last_start.day == start.day:
                        last_array = holder.get_array(last_period)
                        return last_array + int(
                            start.year
                            - last_start.year
                            + (start.month - last_start.month) / 12
                        )

        date_naissance = individu("date_naissance", period)
        epsilon = timedelta64(1)
        return (datetime64(period.start) - date_naissance + epsilon).astype(
            "timedelta64[Y]"
        )


class age_en_mois(Variable):
    value_type = int
    default_value = AGE_INT_MINIMUM
    unit = "months"
    entity = Individu
    label = "Âge (en mois)"
    is_period_size_independent = True
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        # If age_en_mois is known at the same day of another month, compute the new age_en_mois from it.
        holder = individu.get_holder("age_en_mois")
        start = period.start
        known_periods = holder.get_known_periods()

        for last_period in sorted(known_periods, reverse=True):
            last_start = last_period.start
            if last_start.day == start.day:
                last_array = holder.get_array(last_period)
                return last_array + (
                    (start.year - last_start.year) * 12
                    + (start.month - last_start.month)
                )

        has_birth = individu.get_holder("date_naissance").get_known_periods()
        if not has_birth:
            has_age = bool(individu.get_holder("age").get_known_periods())
            if has_age:
                return individu("age", period) * 12
        date_naissance = individu("date_naissance", period)
        epsilon = timedelta64(1)
        return (datetime64(period.start) - date_naissance + epsilon).astype(
            "timedelta64[M]"
        )


class date_naissance(Variable):
    value_type = date
    default_value = date(1970, 1, 1)
    entity = Individu
    label = "Date de naissance"
    definition_period = ETERNITY


class male(Variable):
    value_type = bool
    entity = Individu
    label = "Mâle"
    definition_period = ETERNITY


class marie(Variable):
    value_type = bool
    entity = Individu
    label = "Marié(e)"
    definition_period = YEAR

    def formula(individu, period):
        statut_marital = individu("statut_marital", period=period)
        return statut_marital == 1


class celibataire(Variable):
    value_type = bool
    entity = Individu
    label = "Célibataire"
    definition_period = YEAR

    def formula(individu, period):
        statut_marital = individu("statut_marital", period=period)
        return statut_marital == 2


class divorce(Variable):
    value_type = bool
    entity = Individu
    label = "Divorcé(e)"
    definition_period = YEAR

    def formula(individu, period):
        statut_marital = individu("statut_marital", period=period)
        return statut_marital == 3


class veuf(Variable):
    value_type = bool
    entity = Individu
    label = "Veuf(ve)"
    definition_period = YEAR

    def formula(individu, period):
        statut_marital = individu("statut_marital", period=period)
        return statut_marital == 4


class statut_marital(Variable):
    value_type = int
    default_value = 2
    entity = Individu
    label = "Statut marital"
    definition_period = YEAR



"""
EN: Look into folders to check path length and return an error if a too long path is found.
Also write too long path in a file.

FR: Ce script sert à estimer la longueur des chemins d'arborescence des paramètres,
afin de ne pas avoir de chemins > 150 caractères (incompatible Windows).
Il est à utiliser avant de contribuer à l'harmonisation
"""

import os
import logging


logging.basicConfig(level=logging.INFO)


def extract_paths_too_long(root_dir):
    """
    Look into folders to check path length and return True if a too long path is found.
    """
    path_too_long_detected = False
    maxlen = 0
    root_dir = root_dir.rstrip(os.sep)

    with open("path_too_long_list.txt", "w") as outfile:
        for path, dirs, files in os.walk(root_dir):
            relative_path = path[len(root_dir) + 1 :]
            for dir in dirs:
                if dir.startswith("."):
                    # Ignore hidden directories.
                    dirs.remove(dir)
            for file in files:
                if file.startswith("."):
                    # Ignore hidden files.
                    continue
                relative_file_path = os.path.join(relative_path, file)
                maxlen = max(maxlen, len(relative_file_path))
                if len(relative_file_path) > 150:
                    path_too_long_detected = True
                    logging.error(
                        f"Path too long of {len(relative_file_path) - 150} characters : {relative_file_path}"
                    )
                    outfile.write(
                        "{} here: {}\n".format(
                            len(relative_file_path),
                            relative_file_path,
                        )
                    )
    logging.info(f"Max length found : {maxlen} characters.")
    return path_too_long_detected


country_dir = os.path.dirname((os.path.dirname(os.path.abspath(__file__))))
parameters_dir = os.path.join(country_dir, "parameters")
if extract_paths_too_long(parameters_dir):
    # Return code 1 to indicate an error. Default is 0.
    exit(1)
# If no path are too long, the default return is 0: no error.



from openfisca_tunisia.variables.base import *


class poids(Variable):
    value_type = float
    entity = Individu
    label = "Poids de l'individu dans l'enquête"
    definition_period = YEAR
    is_period_size_independent = True
    default_value = 1


class poids_menage(Variable):
    value_type = float
    entity = Menage
    label = "Poids du ménage dans l'enquête"
    definition_period = YEAR
    is_period_size_independent = True
    default_value = 1


class poids_foyer_fiscal(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Poids du foyer fiscal dans l'enquête"
    definition_period = YEAR
    is_period_size_independent = True
    default_value = 1



from openfisca_tunisia.variables.base import *

# déductions

# 1/ Au titre de l'activité
#
#    Droit commun
# - Déduction de la plus value provenant de l’apport d’actions et de parts sociales au capital de la société mère ou de
#   la société holding 6811
# - Déduction de la plus value provenant de la cession des entreprises en difficultés économiques dans le cadre de la
#   transmission des entreprises 6881
# - Déduction de la plus value provenant de la cession des entreprises suite à l’atteinte du propriétaire de l’âge de la
#   retraite ou à l’incapacité de poursuivre la gestion de l’entreprise dans le cadre de la transmission des
#   entreprises. 6891
# - Déduction de la plus value provenant de l'intégration des éléments d'actifs. 6851


# - Déduction de la plus value provenant de la cession des actions cotées en bourse 6841


class plus_value_cession_actifs_cotes_bourse(Variable):
    value_type = int
    label = "Plus value provenant de la cession des actions cotées en bourse (pour déduction)"
    entity = Individu
    definition_period = YEAR


# - Bénéfices provenant des opérations de courtage international 1141
# - Exportation 1191
# - Location d’immeubles au profit des étudiants 1211 1212 1213
# - Bénéfices provenant des services de restauration au profit des étudiants, des élèves et des apprenants dans les
#   centres de formation professionnelle de base. 1221 1222 1223
# - Bénéfices provenant de la location des constructions verticales destinées à l’habitat collectif social ou
#   économique. 1251
# - Bénéfices provenant de l’exploitation des bureaux d’encadrement et d’assistance fiscale 1311
# - Bénéfices réinvestis dans le capital des sociétés qui commercialisent exclusivement des marchandises ou services
#   tunisiens 1132
# - Bénéfices réinvestis dans les SICAR ou placés auprès d'elles dans des fonds de capital à risque ou dans des fonds
#   de placement à risque qui se conforment aux exigences de l'article 21 de la loi n°: 88-92 relative au sociétés
#   d'investissement. 6872
# - Bénéfices réinvestis dans les SICAR ou placés auprès d'elles dans des fonds de capital à risque ou dans des fonds
#   de placement à risque qui utilisent 75% au moins de leur capital libéré et des montants mis à sa disposition et de
#   leurs actifs dans le financement des projets implantés dans les zones de développement.
# 6842
# - Revenus et bénéfices placés dans les fonds d’amorçage
# 1432
# - Montants déposés dans les comptes épargne pour l’investissement dans la limite de 20000 D
# 1412
# - Montants déposés dans les comptes épargne en actions dans la limite de 20000 D
# 1422
# - Bénéfices réinvestis pour l’acquisition d’entreprises ou de titres cédés suite à l’atteinte du propriétaire de l’âge
#   de la retraite ou à son incapacité de poursuivre la gestion de l’entreprise
# 1512
# - Bénéfices réinvestis pour l’acquisition d’entreprises cédées dans le cadre de cession d’entreprises en difficultés
#   économiques dans le cadre de la loi n° 34 de l'année 1995.
# 1522

#     2/ Autres déductions


class compte_special_epargne_banque(Variable):
    value_type = int
    label = "Intérêts des comptes spéciaux d'épargne ouverts auprès des banques"
    entity = Individu
    definition_period = YEAR


class compte_special_epargne_cent(Variable):
    value_type = int
    label = "Intérêts des comptes spéciaux d'épargne ouverts auprès de la CENT dans la limite"
    entity = Individu
    definition_period = YEAR


class emprunt_obligataire(Variable):
    value_type = int
    label = "Intérêts des emprunts obligataires"
    entity = Individu
    definition_period = YEAR


class compte_epargne_investissement(Variable):
    value_type = int
    label = "Intérêts des comptes épargne pour l'investissement"
    entity = Individu
    definition_period = YEAR


class rente(Variable):
    value_type = int
    label = "Rentes payées obligatoirement et à titre gratuit"
    entity = Individu
    definition_period = YEAR


class prime_assurance_vie(Variable):
    value_type = int
    label = "Prime d’assurance-vie"
    entity = Individu
    definition_period = YEAR


class dons(Variable):
    value_type = int
    label = "Dons au profit du fonds national de solidarité 26-26 et du Fonds National de l’Emploi 21-21"
    entity = Individu
    definition_period = YEAR


class pret_universitaire(Variable):
    value_type = int
    label = "Remboursement des prêts universitaires en principal et intérêts"
    entity = Individu
    definition_period = YEAR


class cotisations_non_affilie(Variable):
    value_type = int
    label = "Cotisations payées par les travailleurs non salariés affiliés à l'un des régimes légaux de la sécurité sociale"
    entity = Individu
    definition_period = YEAR


class interets_acquisition_logement(Variable):
    value_type = int
    label = "Intérêts payés au titre des prêts relatifs à l'acquisition ou à la construction d’un logement social"
    entity = Individu
    definition_period = YEAR


# Code d’incitation aux investissements
# Incitations Communes 3::3
# Bénéfices réinvestis dans l'acquisition des éléments d'actif d'une société ou dans l'acquisition ou la souscription
# d'actions ou de parts permettant de posséder 50% au moins du capital d'une société 2982
# Déduction de 20% des revenus et bénéfices soumis à l'impôt sur le revenu de la part des entreprises dont le chiffre
# d’affaires annuel ne dépasse pas 150 milles dinars pour les activités de services et 300 milles dinars pour les autres
# activités sans dépasser un chiffre d’affaires annuel de 300 milles dinars qui confient la tenue de leurs comptes et la
# préparation de leurs déclarations fiscales aux centres de
# gestion intégrés. (1) 2971
# Exportation totale (pendant la période de la déduction totale). 3222 3223
# Investissement au capital des sociétés de commerce international totalement exportateur. 2172
# Déduction des bénéfices provenant de la gestion d'une zone portuaire destinée au tourisme de croisière (pendant les
# dix premières années à partir de la date d'entrée en activité effective) 2151
# Déduction des bénéfices provenant de la gestion d'une zone portuaire destinée au tourisme de croisière (à partir de
# la onzième année de la date d'entrée en activité effective) 2161
# Déduction des bénéfices réinvestis dans l'acquisition des éléments d'actif d'une société totalement exportatrice ou
# dans l'acquisition d'actions ou de parts permettant de posséder 50% au moins du capital d'une société totalement
# exportatrice dans le cadre de la loi n° 34 de l'année 1995. 2142
# Exportation partielle. 3232
# Développement régional: le premier groupe 3461 3463
# Développement régional: le deuxième groupe 2371 2372
# Développement régional prioritaire pendant les dix premières années à partir de la date d'entrée en activité effective
#  ( 2) 2391 2392
# Développement régional prioritaire pendant les dix années qui suivent des dix premières années à partir de la date
# d'entrée en activité effective ( 2) 2381 2382
# Déduction des bénéfices réinvestis dans l'acquisition des éléments d'actif des sociétés exerçant dans les zones
# d'encouragement au développement régional ou dans l'acquisition ou la souscription d'actions ou de parts permettant
# de posséder 50% au moins du capital de ces sociétés dans le cadre de la loi n° 34 de l'année 1995. 2352
# Travaux publics et promotion immobilière dans la zone de développement régional . 3422
# Développement agricole 35:2 35:3
# Investissements agricoles réalisés dans les régions à climat difficile ainsi que les investissements de pêche dans
# les zones insuffisamment exploitées 3523
# Lutte contre la pollution 38:2 38:3
# Activités de soutien 33:2 33:3
# Bénéfices provenant de projets réalisés par les promoteurs immobiliers concernant les programmes de logements sociaux
# et de réaménagement des zones d’activités agricoles, touristiques, industrielles et les bâtiments pour les activités
# industrielles.
# 36:2
# Sociétés implantées dans les parcs des activités économiques
# 4262 4263
# Bénéfices et revenus réinvestis dans le cadre de la mise à niveau des entreprises publiques.
#



"""3. Bénéfices de l'exploitation agricole et de pêche."""

from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class beap(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Bénéfices de l'exploitation agricole et de pêche (BEAP)"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        beap_reel_res_fiscal = foyer_fiscal.sum(
            foyer_fiscal.members("beap_reel_res_fiscal", period=period)
        )
        beap_reliq_benef_fiscal = foyer_fiscal.sum(
            foyer_fiscal.members("beap_reliq_benef_fiscal", period=period)
        )
        beap_monogr = foyer_fiscal.sum(
            foyer_fiscal.members("beap_monogr", period=period)
        )
        beap_part_benef_sp = foyer_fiscal.sum(
            foyer_fiscal.members("beap_part_benef_sp", period=period)
        )

        return (
            beap_reel_res_fiscal
            + beap_reliq_benef_fiscal
            + beap_monogr
            + beap_part_benef_sp
        )



import argparse
import requests
import logging


logging.basicConfig(level=logging.INFO)


def get_info(package_name: str = "") -> dict:
    """
    Get minimal informations needed by .conda/meta.yaml from PyPi JSON API.
    ::package_name:: Name of package to get infos from.
    ::return:: A dict with last_version, url and sha256
    """
    if package_name == "":
        raise ValueError("Package name not provided.")
    resp = requests.get(f"https://pypi.org/pypi/{package_name}/json").json()
    version = resp["info"]["version"]
    for v in resp["releases"][version]:
        if v["packagetype"] == "sdist":  # for .tag.gz
            return {
                "last_version": version,
                "url": v["url"],
                "sha256": v["digests"]["sha256"],
            }


def replace_in_file(filepath: str, info: dict):
    """
    ::filepath:: Path to meta.yaml, with filename
    ::info:: Dict with information to populate
    """
    with open(filepath, "rt") as fin:
        meta = fin.read()
    # Replace with info from PyPi
    meta = meta.replace("PYPI_VERSION", info["last_version"])
    meta = meta.replace("PYPI_URL", info["url"])
    meta = meta.replace("PYPI_SHA256", info["sha256"])
    with open(filepath, "wt") as fout:
        fout.write(meta)
    logging.info(f"File {filepath} has been updated with informations from PyPi.")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-p",
        "--package",
        type=str,
        default="",
        required=True,
        help="The name of the package",
    )
    parser.add_argument(
        "-f",
        "--filename",
        type=str,
        default=".conda/meta.yaml",
        help="Path to meta.yaml, with filename",
    )
    args = parser.parse_args()
    info = get_info(args.package)
    logging.info(f"Information of the last published PyPi package : {info}")
    replace_in_file(args.filename, info)



from openfisca_core.entities import build_entity


Individu = build_entity(
    key="individu", plural="individus", label="Individu", is_person=True
)

FoyerFiscal = build_entity(
    key="foyer_fiscal",
    plural="foyers_fiscaux",
    label="Foyer fiscal",
    roles=[
        {
            "key": "declarant",
            "plural": "declarants",
            "label": "Déclarants",
            "subroles": ["declarant_principal", "conjoint"],
        },
        {
            "key": "personne_a_charge",
            "plural": "personnes_a_charge",
            "label": "Personnes à charge",
        },
    ],
)

Menage = build_entity(
    key="menage",
    plural="menages",
    label="Ménage",
    roles=[
        {"key": "personne_de_reference", "label": "Personne de référence", "max": 1},
        {"key": "conjoint", "label": "Conjoint", "max": 1},
        {"key": "enfant", "plural": "enfants", "label": "Enfants"},
        {"key": "ascendant", "plural": "ascendants", "label": "Ascendants"},
        {"key": "autre", "plural": "autres", "label": "Autres"},
    ],
)

entities = [Individu, FoyerFiscal, Menage]



"""6. Revenus de valeurs mobilières et de capitaux mobiliers."""

from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class tspr(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Traitements, salaires, indemnités, pensions et rentes viagères"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        revenu_assimile_salaire_apres_abattements = foyer_fiscal(
            "revenu_assimile_salaire_apres_abattements", period=period
        )
        revenu_assimile_pension_apres_abattements = foyer_fiscal(
            "revenu_assimile_pension_apres_abattements", period=period
        )

        return (
            revenu_assimile_salaire_apres_abattements
            + revenu_assimile_pension_apres_abattements
        )


class revenu_assimile_salaire(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Revenu assimilé à des salaires"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        salaire_imposable = foyer_fiscal.declarant_principal(
            "salaire_imposable", period=period, options=[ADD]
        )
        salaire_en_nature = foyer_fiscal.declarant_principal(
            "salaire_en_nature", period=period, options=[ADD]
        )
        return salaire_imposable + salaire_en_nature


class smig(Variable):
    value_type = bool
    entity = FoyerFiscal
    label = "Indicatrice de SMIG ou SMAG déduite du montant des salaires"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        revenu_assimile_salaire = foyer_fiscal("revenu_assimile_salaire", period=period)
        salarie_declarant_percevoir_smig = foyer_fiscal.declarant_principal(
            "salarie_declarant_percevoir_smig", period=period.first_month
        )
        smig_40h_mensuel = parameters(period.start).marche_travail.smig_40h_mensuel
        smig = salarie_declarant_percevoir_smig + (
            revenu_assimile_salaire <= 12 * smig_40h_mensuel
        )
        return smig


class revenu_assimile_salaire_apres_abattements(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Revenu imposé comme des salaires net des abatements"
    definition_period = YEAR

    def formula_2011(foyer_fiscal, period, parameters):
        revenu_assimile_salaire = foyer_fiscal("revenu_assimile_salaire", period=period)
        smig = foyer_fiscal("smig", period=period)
        tspr = parameters(period.start).impot_revenu.tspr
        abattement_frais_professionnels = min_(
            revenu_assimile_salaire * tspr.abat_sal, tspr.max_abat_sal
        )
        revenu_assimile_salaire_apres_abattements = max_(
            (
                revenu_assimile_salaire
                - abattement_frais_professionnels
                - max_(
                    smig * tspr.abattement_pour_salaire_minimum,
                    (revenu_assimile_salaire <= tspr.smig_ext)
                    * tspr.abattement_pour_salaire_minimum,
                )
            ),
            0,
        )
        return revenu_assimile_salaire_apres_abattements

    def formula(foyer_fiscal, period, parameters):
        revenu_assimile_salaire = foyer_fiscal("revenu_assimile_salaire", period=period)
        smig = foyer_fiscal("smig", period=period)
        tspr = parameters(period.start).impot_revenu.tspr
        abattement_frais_professionnels = min_(
            revenu_assimile_salaire * tspr.abat_sal, tspr.max_abat_sal
        )
        revenu_assimile_salaire_apres_abattements = min_(
            revenu_assimile_salaire
            - abattement_frais_professionnels
            - smig * tspr.abattement_pour_salaire_minimum,
            0,
        )
        return revenu_assimile_salaire_apres_abattements


class revenu_assimile_pension_apres_abattements(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Revenu assimilé à des pensions après abattements"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        revenu_assimile_pension = foyer_fiscal.declarant_principal(
            "revenu_assimile_pension", period=period
        )
        avantages_nature_assimile_pension = foyer_fiscal.declarant_principal(
            "avantages_nature_assimile_pension", period=period
        )
        tspr = parameters(period.start).impot_revenu.tspr
        return (revenu_assimile_pension + avantages_nature_assimile_pension) * (
            1 - tspr.abat_pen
        )



# OpenFisca -- A versatile microsimulation software
# By: OpenFisca Team <contact@openfisca.fr>
#
# Copyright (C) 2011, 2012, 2013, 2014, 2015 OpenFisca Team
# https://github.com/openfisca
#
# This file is part of OpenFisca.
#
# OpenFisca is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# OpenFisca is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""Conversion functions"""

from openfisca_core.conv import *


json_to_natural_int = pipe(
    test_isinstance(int),
    test_greater_or_equal(0),
)


def test_in_pop(values, error=None):
    def remove(value):
        values.remove(value)
        return value

    return pipe(
        test_in(values, error=error),
        function(remove),
    )



from openfisca_tunisia.variables.base import *


# Revenus fonciers


# Régime réel


class foncier_reel_resultat_fiscal(Variable):
    value_type = int
    entity = Individu
    label = "Résultat fiscal (revenus fonciers, régime réel)"
    definition_period = YEAR


# Régime forfaitaire bâti


class foncier_forfaitaire_batis_recettes(Variable):
    value_type = int
    entity = Individu
    label = "Recettes brutes réalisées TTC (revenus fonciers, base forfaitaire, immeubles bâtis)"
    definition_period = YEAR


class foncier_forfaitaire_batis_reliquat(Variable):
    value_type = int
    entity = Individu
    label = "Reliquat (revenus fonciers, base forfaitaire, immeubles bâtis)"
    definition_period = YEAR


class foncier_forfaitaire_batis_frais(Variable):
    value_type = int
    entity = Individu
    label = "Frais d'entretien et de réparation justifiés à déduire (revenus fonciers, base forfaitaire, immeubles bâtis)"
    definition_period = YEAR


class foncier_forfaitaire_batis_taxe(Variable):
    value_type = int
    entity = Individu
    label = "Taxe effectivement payée à déduire (revenus fonciers, base forfaitaire, immeubles bâtis)"
    definition_period = YEAR


# Régime forfaitaire non bâti


class foncier_forfaitaire_non_batis_recettes(Variable):
    value_type = int
    entity = Individu
    label = "Recettes brutes réalisées TTC (revenus fonciers, base forfaitaire, terrains non bâtis)"
    definition_period = YEAR


class foncier_forfaitaire_non_batis_depenses(Variable):
    value_type = int
    entity = Individu
    label = "Dépenses justifiées à déduire (revenus fonciers, base forfaitaire, terrains non bâtis)"
    definition_period = YEAR


class foncier_forfaitaire_non_batis_taxe(Variable):
    value_type = int
    entity = Individu
    label = "Taxe effectivement payée à déduire (revenus fonciers, base forfaitaire, terrains non bâtis)"
    definition_period = YEAR


class foncier_societes_personnes(Variable):
    value_type = int
    entity = Individu
    label = "Part dans le bénéfice ou dans la perte des sociétés de personnes et assimilées qui réalisent des revenus fonciers"
    definition_period = YEAR



from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class impots_directs(Variable):
    value_type = float
    entity = Individu
    label = "Impôts directs"
    definition_period = YEAR

    def formula(individu, period):
        irpp = individu.foyer_fiscal("irpp", period=period)

        return irpp


class prestations_sociales(Variable):
    value_type = int
    entity = Individu
    label = "Prestations sociales"
    definition_period = YEAR


class revenu_disponible(Variable):
    value_type = float
    entity = Menage
    label = "Revenu disponible du ménage"
    definition_period = YEAR

    def formula(menage, period):
        revenu_disponible_individuels = menage.members(
            "revenu_disponible_individuel", period=period
        )
        return menage.sum(revenu_disponible_individuels)


class revenu_disponible_individuel(Variable):
    value_type = float
    entity = Individu
    label = "Revenu disponible individuel"
    definition_period = YEAR

    def formula(individu, period):
        revenus_du_travail = individu("revenus_du_travail", period=period)
        revenu_assimile_pension = individu("revenu_assimile_pension", period=period)
        revenus_du_capital = individu("revenus_du_capital", period=period)
        prestations_sociales = individu("prestations_sociales", period=period)
        impots_directs = individu("impots_directs", period=period)

        return (
            revenus_du_travail
            + revenu_assimile_pension
            + revenus_du_capital
            + prestations_sociales
            + impots_directs
        )


class revenus_du_capital(Variable):
    value_type = float
    entity = Individu
    label = "Revenus du capital"
    definition_period = YEAR

    def formula(individu, period):
        revenus_fonciers = individu.foyer_fiscal("revenus_fonciers", period=period)
        return revenus_fonciers


class revenus_du_travail(Variable):
    value_type = float
    entity = Individu
    label = "Revenus du travail"
    definition_period = YEAR

    def formula(individu, period):
        salaire_imposable = individu("salaire_imposable", period=period, options=[ADD])
        contribution_sociale_solidarite = individu(
            "contribution_sociale_solidarite_prelevee_a_la_source",
            period=period,
            options=[ADD],
        )
        return (
            salaire_imposable - contribution_sociale_solidarite
        )  # + beap + bic + bnc  TODO



"""Revenus de valeurs mobilières et de capitaux mobiliers."""

from openfisca_tunisia.variables.base import *


# A Revenus des valeurs mobilières et de capitaux mobiliers


class valm_nreg(Variable):
    value_type = int
    label = "Revenus des valeurs mobilières autres que ceux régulièrement distribués"
    entity = Individu
    definition_period = YEAR


class valm_jpres(Variable):
    value_type = int
    label = "Jetons de présence"
    entity = Individu
    definition_period = YEAR


class valm_aut(Variable):
    value_type = int
    label = "Autres rémunérations assimilées"
    entity = Individu
    definition_period = YEAR


# B Revenus de capitaux mobiliers


class capm_banq(Variable):
    value_type = int
    label = "Intérêts bruts des comptes spéciaux d'épargne ouverts auprès des banques"
    entity = Individu
    definition_period = YEAR


class capm_cent(Variable):
    value_type = int
    label = "Intérêts bruts des comptes spéciaux d'épargne ouverts auprès de la CENT"
    entity = Individu
    definition_period = YEAR


class capm_caut(Variable):
    value_type = int
    label = "Intérêts des créances et intérêts et rémunérations des cautionnements"
    entity = Individu
    definition_period = YEAR


class capm_part(Variable):
    value_type = int
    label = "Intérêts des titres de participation"
    entity = Individu
    definition_period = YEAR


class capm_oblig(Variable):
    value_type = int
    label = "Intérêts des emprunts obligataires"
    entity = Individu
    definition_period = YEAR


class capm_caisse(Variable):
    value_type = int
    label = "Intérêts des bons de caisse"
    entity = Individu
    definition_period = YEAR


class capm_plfcc(Variable):
    value_type = int
    label = "Revenus des parts et de liquidation du fonds commun des créances"
    entity = Individu
    definition_period = YEAR


class capm_epinv(Variable):
    value_type = int
    label = "Intérêts des comptes épargne pour l'investissement"
    entity = Individu
    definition_period = YEAR


class capm_aut(Variable):
    value_type = int
    label = "Autres intérêts"
    entity = Individu
    definition_period = YEAR



"""2. Bénéfices des professions non commerciales"""

from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class bnc(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Bénéfices des professions non commerciales (BNC)"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        bnc_reel_res_fiscal = foyer_fiscal.sum(
            foyer_fiscal.members("bnc_reel_res_fiscal", period=period)
        )
        bnc_forfaitaire_benefice_fiscal = foyer_fiscal.sum(
            foyer_fiscal.members("bnc_forfaitaire_benefice_fiscal", period=period)
        )
        bnc_part_benef_sp = foyer_fiscal.sum(
            foyer_fiscal.members("bnc_part_benef_sp", period=period)
        )
        return bnc_reel_res_fiscal + bnc_forfaitaire_benefice_fiscal + bnc_part_benef_sp


class bnc_forfaitaire_benefice_fiscal(Variable):
    value_type = float
    entity = Individu
    label = "Bénéfice fiscal (régime forfaitaire en % des recettes brutes TTC)"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        """
        Bénéfice fiscal (régime forfaitaire, en % des recettes brutes TTC)
        """
        bnc_forfaitaire_recettes_brutes = foyer_fiscal(
            "bnc_forfaitaire_recettes_brutes", period=period
        )
        part = parameters(period.start).impot_revenu.bnc.forf.part_forf
        return bnc_forfaitaire_recettes_brutes * part



import datetime

from openfisca_tunisia import TunisiaTaxBenefitSystem


# Exceptionally for this test do not import TaxBenefitSystem from tests.base.
tax_benefit_system = TunisiaTaxBenefitSystem()


def test_parameters():
    parameters = tax_benefit_system.parameters
    assert parameters is not None
    for year in range(2006, datetime.date.today().year + 1):
        parameters_at_instant = tax_benefit_system.get_parameters_at_instant(year)
        assert parameters_at_instant is not None



from openfisca_tunisia.variables.base import *

try:
    from scipy.optimize import fsolve
except ImportError:
    fsolve = None


def calculate_net_from(salaire_imposable, individu, period, requested_variable_names):
    # Work in isolation
    temp_simulation = individu.simulation.clone()
    temp_individu = temp_simulation.individu

    # Calculated variable holders might contain undesired cache
    # (their entity.simulation points to the original simulation above)
    for name in requested_variable_names:
        temp_individu.get_holder(name).delete_arrays(period)

    # We're not wanting to calculate salaire_imposable again,
    # but instead manually set it as an input variable
    temp_individu.get_holder("salaire_imposable").set_input(period, salaire_imposable)

    # Force recomputing of salaire_net_a_payer
    temp_individu.get_holder("salaire_net_a_payer").delete_arrays(period)
    net = temp_individu("salaire_net_a_payer", period)[0]

    return net


class salaire_imposable(Variable):
    value_type = float
    entity = Individu
    label = "Salaire imposable"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        # Use numerical inversion to calculate 'salaire_imposable' from 'salaire_net_a_payer'
        net = individu.get_holder("salaire_net_a_payer").get_array(period)
        if net is None:
            return individu.empty_array()

        simulation = individu.simulation
        simulation.period = period

        # List of variables already calculated.
        # We will need it to remove their holders, that might contain undesired cache
        requested_variable_names = [
            stack_frame["name"] for stack_frame in individu.simulation.tracer.stack
        ]

        def solve_func(net):
            def innerfunc(essai):
                return (
                    calculate_net_from(
                        essai, individu, period, requested_variable_names
                    )
                    - net
                )

            return innerfunc

        imposable_calcule = fsolve(
            solve_func(net),
            net * 1.25,  # on entend souvent parler cette méthode...
            xtol=1 / 100,  # précision
        )

        return imposable_calcule


class de_net_a_imposable(Reform):
    name = "Inversion du calcul imposable -> net"

    def apply(self):
        self.update_variable(salaire_imposable)



from openfisca_tunisia.tunisia_taxbenefitsystem import TunisiaTaxBenefitSystem

# The name CountryTaxBenefitSystem must not be changed, as all tools of the OpenFisca
# ecosystem expect a CountryTaxBenefitSystem class to be exposed
# in the __init__ module of a country package.

CountryTaxBenefitSystem = TunisiaTaxBenefitSystem



import logging

log = logging.getLogger(__name__)


def init_single_entity(
    scenario,
    axes=None,
    enfants=None,
    foyer_fiscal=None,
    menage=None,
    parent1=None,
    parent2=None,
    period=None,
):
    if enfants is None:
        enfants = []
    assert parent1 is not None

    foyers_fiscaux = {}
    menages = {}
    individus = {}

    count_so_far = 0
    for nth in range(0, 1):
        foyer_fiscal_nth = foyer_fiscal.copy() if foyer_fiscal is not None else {}
        menage_nth = menage.copy() if menage is not None else {}
        group = [parent1, parent2] + (enfants or [])
        for index, individu in enumerate(group):
            if individu is None:
                continue
            id = individu.get("id")
            if id is None:
                individu = individu.copy()
                id = "ind{}".format(index + count_so_far)
            individus[id] = individu
            if index <= 1:
                foyer_fiscal_nth.setdefault("declarants", []).append(id)
                if index == 0:
                    menage_nth["personne_de_reference"] = id
                else:
                    menage_nth["conjoint"] = id
            else:
                foyer_fiscal_nth.setdefault("personnes_a_charge", []).append(id)
                menage_nth.setdefault("enfants", []).append(id)

        count_so_far += len(group)
        foyers_fiscaux["ff{}".format(nth)] = foyer_fiscal_nth
        menages["m{}".format(nth)] = menage_nth

    dict = {
        "period": period,
        "foyers_fiscaux": foyers_fiscaux,
        "menages": menages,
        "individus": individus,
    }
    if axes:
        dict["axes"] = axes
    scenario.init_from_dict(dict)
    return scenario



from openfisca_tunisia.variables.base import *


class boursier(Variable):
    value_type = bool
    entity = Individu
    definition_period = YEAR


class eleve(Variable):
    value_type = bool
    entity = Individu
    definition_period = YEAR


class etudiant(Variable):
    value_type = bool
    entity = Individu
    definition_period = YEAR



"""4. Revenus fonciers."""

from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class revenus_fonciers(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Revenus fonciers"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        foncier_reel_resultat_fiscal = foyer_fiscal.declarant_principal(
            "foncier_reel_resultat_fiscal", period=period
        )
        foncier_forfait_bati_resultat_fiscal = foyer_fiscal(
            "foncier_forfait_bati_resultat_fiscal", period=period
        )
        foncier_forfait_non_bati_resultat_fiscal = foyer_fiscal(
            "foncier_forfait_non_bati_resultat_fiscal", period=period
        )
        foncier_societes_personnes = foyer_fiscal.declarant_principal(
            "foncier_societes_personnes", period=period
        )

        return (
            foncier_reel_resultat_fiscal
            + foncier_forfait_bati_resultat_fiscal
            + foncier_forfait_non_bati_resultat_fiscal
            + foncier_societes_personnes
        )


class foncier_forfait_bati_resultat_fiscal(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Revenus fonciers net des immeubles bâtis"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        foncier_forfaitaire_batis_recettes = foyer_fiscal.declarant_principal(
            "foncier_forfaitaire_batis_recettes", period=period
        )
        # foncier_forfaitaire_batis_reliquat = foyer_fiscal.declarant_principal(
        #     'foncier_forfaitaire_batis_reliquat', period = period)
        foncier_forfaitaire_batis_frais = foyer_fiscal.declarant_principal(
            "foncier_forfaitaire_batis_frais", period=period
        )
        foncier_forfaitaire_batis_taxe = foyer_fiscal.declarant_principal(
            "foncier_forfaitaire_batis_taxe", period=period
        )
        taux_deduction_frais = parameters(
            period.start
        ).impot_revenu.foncier.bati.deduction_frais
        return max_(
            0,
            foncier_forfaitaire_batis_recettes * (1 - taux_deduction_frais)
            - foncier_forfaitaire_batis_frais
            - foncier_forfaitaire_batis_taxe,
        )


class foncier_forfait_non_bati_resultat_fiscal(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Revenus fonciers net des terrains non bâtis"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        foncier_forfaitaire_non_batis_recettes = foyer_fiscal.declarant_principal(
            "foncier_forfaitaire_non_batis_recettes", period=period
        )
        foncier_forfaitaire_non_batis_depenses = foyer_fiscal.declarant_principal(
            "foncier_forfaitaire_non_batis_depenses", period=period
        )
        foncier_forfaitaire_non_batis_taxe = foyer_fiscal.declarant_principal(
            "foncier_forfaitaire_non_batis_taxe", period=period
        )
        return max_(
            foncier_forfaitaire_non_batis_recettes
            - foncier_forfaitaire_non_batis_depenses
            - foncier_forfaitaire_non_batis_taxe,
            0,
        )



import re

# This script prints the minimal version of Openfisca-Core to ensure their compatibility during CI testing.
# Uses only the standard library (no tomli) so it runs in CI before dependencies are installed.
# When pyproject has ">=44, <45", we must output a concrete version that exists on PyPI (e.g. 44.0.3).
MINIMAL_PATCH = {
    "44": "44.0.3"
}  # first available 44.x on PyPI (44.0.0–44.0.2 not published)

with open("./pyproject.toml", "r", encoding="utf-8") as file:
    content = file.read()

match = re.search(
    r"openfisca-core\[([^\]]+)\]\s*>=\s*([\d\.]+)",
    content,
)
if not match:
    raise SystemExit("Could not find openfisca-core dependency in pyproject.toml")

extra, min_ver = match[1], match[2]
install_ver = MINIMAL_PATCH.get(min_ver, min_ver)
print(f"openfisca-core[{extra}]=={install_ver}")  # noqa: T201



from openfisca_tunisia.variables.base import *


class amen_social_presence_handicap_lourd(Variable):
    value_type = bool
    entity = Menage
    label = "Ménage comprenant un membre avec un handicap lourd"
    definition_period = MONTH
    # Critères primaires du décret 2020-317 du 19 mai 2020
    # Handicap de niveau 3

    def formula_2020(menage, period, parameters):
        return menage.sum(menage.members("handicap", period) >= 3)


class amen_social_pas_d_achat_onereux(Variable):
    value_type = bool
    default_value = True
    entity = Menage
    label = "Ménage n'ayant pas fait d'achat onéreux"
    definition_period = ETERNITY
    # Critères primaires du décret 2020-317 du 19 mai 2020
    # Ni le chef du ménage ni aucun membre de son ménage n’a effectué une transaction d’achat
    # ou de vente d’un bien mobilier ou immobilier dont la valeur dépasse 30 fois le SMIG. Ce
    # critère est vérifiable actuellement uniquement pour les véhicules immatriculés par les
    # recoupements administratifs avec l’Agence Technique des Transports Terrestres (ATTT).


class amen_social_pas_de_residence_secondaire(Variable):
    value_type = bool
    default_value = True
    entity = Menage
    label = "Ménage ne possédant pas de résidence secondaire"
    definition_period = ETERNITY
    # Critères primaires du décret 2020-317 du 19 mai 2020
    # Le ménage n’est pas propriétaire d’un logement secondaire


class amen_social_eligible(Variable):
    value_type = bool
    entity = Menage
    label = "Ménage éligible au programme Amen social"
    definition_period = MONTH

    def formula_2020(menage, period, parameters):
        # Critères primaires du décret 2020-317 du 19 mai 2020
        pas_de_residence_secondaire = menage(
            "amen_social_pas_de_residence_secondaire", period
        )
        pas_d_achat_onereux = menage("amen_social_pas_d_achat_onereux", period)

        # Citère du revenu selon présence ou non de handicap lourd
        presence_handicap_lourd = menage("amen_social_presence_handicap_lourd", period)
        taille_menage = menage.nb_persons()
        revenu_menage = menage("amen_social_revenu", period)
        seuil_de_revenu = parameters(
            period
        ).prestations.non_contributives.amen_social.eligibilite
        smig_mensuel = parameters(period.start).marche_travail.smig_48h_mensuel
        conditions_sans_handicap = [
            taille_menage == 1,
            taille_menage == 2,
            (taille_menage == 3) + (taille_menage == 4),
            taille_menage >= 5,
        ]
        valeurs_choisies_sans_handicap = [
            smig_mensuel * seuil_de_revenu.un_membre,
            smig_mensuel * seuil_de_revenu.deux_membres,
            smig_mensuel * seuil_de_revenu.trois_quatre_membres,
            smig_mensuel * seuil_de_revenu.plus_de_cinq_membres,
        ]

        conditions_avec_handicap = [
            (taille_menage == 1) * presence_handicap_lourd,
            (taille_menage == 2) * presence_handicap_lourd,
            ((taille_menage == 3) + (taille_menage == 4)) * presence_handicap_lourd,
            (taille_menage >= 5) * presence_handicap_lourd,
        ]
        valeurs_choisies_avec_handicap = [
            smig_mensuel * seuil_de_revenu.handicap_lourd.un_membre,
            smig_mensuel * seuil_de_revenu.handicap_lourd.deux_membres,
            smig_mensuel * seuil_de_revenu.handicap_lourd.trois_quatre_membres,
            smig_mensuel * seuil_de_revenu.handicap_lourd.plus_de_cinq_membres,
        ]

        critere_revenu = where(
            presence_handicap_lourd,
            revenu_menage
            <= select(conditions_avec_handicap, valeurs_choisies_avec_handicap),
            revenu_menage
            <= select(conditions_sans_handicap, valeurs_choisies_sans_handicap),
        )
        return pas_d_achat_onereux * pas_de_residence_secondaire * critere_revenu


class amen_social_revenu(Variable):
    value_type = float
    entity = Menage
    label = "Revenu du ménage au sens du programme Amen social"
    definition_period = MONTH

    def formula_2020(menage, period, parameters):
        revenu_menage = menage.sum(
            menage.members("salaire_net_a_payer", period)
        ) + menage.sum(menage.members("pension_de_retraite", period))
        return revenu_menage


class transfert_monetaire_permanent_eligible(Variable):
    value_type = bool
    entity = Menage
    label = "Ménage éligible au transfert monétaire permanent du programme Amen social"
    definition_period = MONTH

    def formula_2020(menage, period):
        return menage(
            "transfert_monetaire_permanent_eligibilite_score", period
        ) + menage("transfert_monetaire_permanent_eligibilite_supplementaire", period)


class transfert_monetaire_permanent_eligibilite_score(Variable):
    value_type = bool
    entity = Menage
    label = "Ménage éligible au programme Amen social selon le modèle de ciblage"
    definition_period = MONTH

    def formula_2022_06(menage, period, parameters):
        decile = parameters(period).prestations.non_contributives.amen_social.decile
        return menage("amen_social_eligible", period) + (
            menage("amen_social_score_decile", period) <= decile
        )


class transfert_monetaire_permanent_eligibilite_supplementaire(Variable):
    value_type = bool
    entity = Menage
    label = "Ménage éligible au programme Amen social selon les critères supplémentaires d'éligibilité"
    definition_period = MONTH


class amen_social_enfants_a_charge(Variable):
    value_type = int
    entity = Menage
    label = "Enfant considéré à charge au sens de la prestation Amen social"
    definition_period = MONTH

    def formula_2020(menage, period, parameters):
        age = menage.members("age", period)
        # Les enfants sont considérés comme elève dès 6 ans  (pris quand ?)
        # eleve = menage.members('eleve', period.this_year)
        etudiant = menage.members("etudiant", period.this_year)
        handicap = menage.members("handicap", period.this_year) == 3
        amen_social = parameters(
            period
        ).prestations.non_contributives.amen_social.supplements
        condition_enfant = (age >= 6) * (
            age <= amen_social.limite_age_enfant
        )  # * eleve
        condition_jeune_etudiant = (age <= amen_social.limite_age_etudiant) * etudiant
        enfant_a_charge = condition_enfant + condition_jeune_etudiant + handicap
        return menage.sum(1 * enfant_a_charge, role=Menage.ENFANT)


class amen_social_enfants_handicapes_a_charge(Variable):
    value_type = int
    entity = Menage
    label = "Enfant considéré à charge au sens de la prestation Amen social"
    definition_period = MONTH

    def formula_2020(menage, period, parameters):
        age = menage.members("age", period)
        handicap = menage.members("handicap", period.this_year) == 3
        amen_social = parameters(
            period
        ).prestations.non_contributives.amen_social.supplements
        condition_enfant = (age >= 6) * (age <= amen_social.limite_age_enfant)
        condition_jeune_etudiant = age <= amen_social.limite_age_etudiant
        enfant_a_charge_handicape = (
            condition_enfant + condition_jeune_etudiant
        ) * handicap
        return menage.sum(enfant_a_charge_handicape, role=Menage.ENFANT)


class transfert_monetaire_permanent(Variable):
    value_type = float
    entity = Menage
    label = "Transfert monétaire permanent mensuel du programme Amen social"
    definition_period = MONTH

    def formula_2020(menage, period, parameters):
        amen_social = parameters(period).prestations.non_contributives.amen_social
        supplements = amen_social.supplements
        eligible = menage("transfert_monetaire_permanent_eligible", period)
        enfants_a_charge = menage("amen_social_enfants_a_charge", period)
        enfants_handicapes_a_charge = menage(
            "amen_social_enfants_handicapes_a_charge", period
        )
        tmp = eligible * (
            amen_social.allocation_base
            + enfants_a_charge * supplements.amen_social_enfants_a_charge
            + enfants_handicapes_a_charge
            * supplements.amen_social_enfants_a_charge
            * supplements.handicap
        )
        return tmp


# TODO

# - séparer éligibilité AMEN et TMP
# - Coder éligibilité AMG1 et AMG2

# Amen regroupe amg1 amg2 et tmp

# sante_amg1

# sante_amg2
# - Limiter  Personne à charge : que enfant à charge
#
# Enfant en âge de scolairite que par âge 6- 18 an sauf demande attestaion pour rentrée scolaire
# pour prestation monétaire ou en nature

# - Rajouter allocations familiales
# Allocaion familiales
# Critère d'éligibilité inscrit à Amen et IS et décret 3.17 valide.
# 30 DT - de 0 à 18.
# Pas de score


# - Niveau de handicap
# handicap_leger, handicap_lourd



from openfisca_tunisia.variables.base import *  # noqa F401
from openfisca_tunisia.scenarios import init_single_entity
from tests import base

import datetime


def test_contribution_frais_creche():
    year = 2011
    month = "2011-01"
    simulation = init_single_entity(
        base.tax_benefit_system.new_scenario(),
        period=year,
        parent1=dict(age=40),
        parent2=dict(
            date_naissance=datetime.date(1972, 1, 1),
        ),
        enfants=[
            dict(
                # id = 'Behija',
                date_naissance="1992-01-01",
            ),
            dict(
                # id = 'Hassen',
                date_naissance="2002-01-01",
            ),
            dict(
                # id = 'Thoura',
                date_naissance="2008-12-31",
            ),
        ],
    ).new_simulation(debug=True)

    ages_en_mois = simulation.menage.members("age_en_mois", period=month)
    age_benjamin = simulation.menage.min(ages_en_mois)[0]
    assert age_benjamin == 24
    assert (
        age_benjamin
        <= base.tax_benefit_system.parameters(
            year
        ).prestations.contributives.prestations_familiales.creche.age_max
    )

    contribution_frais_creche = simulation.calculate(
        "contribution_frais_creche", period=year
    )
    assert contribution_frais_creche != 0


if __name__ == "__main__":
    test_contribution_frais_creche()



from openfisca_tunisia.variables.base import *


# Salaires


class salaire_de_base(Variable):
    value_type = float
    label = "Salaire de base"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period


class primes(Variable):
    value_type = float
    label = "Primes"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period


class salaire_en_nature(Variable):
    value_type = float
    label = "Avantages en nature assimilables à des salaires"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period


class salarie_declarant_percevoir_smig(Variable):
    value_type = bool
    label = "Salarié déclarant percevoir le SMIG ou le SMAG"
    entity = Individu
    definition_period = MONTH



from openfisca_tunisia.variables.base import *


# Pensions


class pension_cnss(Variable):
    value_type = float
    label = "Pension"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period


class pension_cnrps(Variable):
    value_type = float
    label = "Pension"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period


class pension_de_retraite(Variable):
    value_type = float
    label = "Pension"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period


class pension_orphelin(Variable):
    value_type = float
    label = "Pension"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period


class pension_d_invalidite(Variable):
    value_type = float
    label = "Pension"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period


class pension(Variable):
    value_type = float
    label = "Pension"
    entity = Individu
    definition_period = YEAR
    set_input = set_input_divide_by_period

    def formula(individu, period):
        return (
            individu("pension_de_retraite", period, options=[ADD])
            + individu("pension_d_invalidite", period, options=[ADD])
            + individu("pension_orphelin", period, options=[ADD])
        )


class revenu_assimile_pension(Variable):
    value_type = float
    label = "Revenus assimilés à des pensions (pensions et rentes viagères)"
    entity = Individu
    definition_period = YEAR

    def formula_2025_01(individu, period):
        return individu("pension_de_retraite", period, options=[ADD])

    def formula(individu, period):
        return individu("pension", period, options=[ADD])


class avantages_nature_assimile_pension(Variable):
    value_type = float
    label = "Avantages en nature assimilables à des pensions"
    entity = Individu
    definition_period = YEAR



# Read package version in pyproject.toml and replace it in .conda/recipe.yaml

import argparse
import logging
import re

logging.basicConfig(level=logging.INFO, format="%(message)s")
PACKAGE_VERSION = "X.X.X"
CORE_VERSION = ">=43,<44"
NUMPY_VERSION = ">=1.24.3,<2"


def get_versions():
    """
    Read package version and deps in pyproject.toml
    """
    openfisca_core_api = None
    openfisca_tunisia = None
    with open("./pyproject.toml", "r") as file:
        content = file.read()
    # Extract the version of openfisca_tunisia
    version_match = re.search(r'^version\s*=\s*"([\d.]*)"', content, re.MULTILINE)
    if version_match:
        openfisca_tunisia = version_match.group(1)
    else:
        raise Exception("Package version not found in pyproject.toml")
    # Extract dependencies
    version = re.search(
        r'openfisca-core\[web-api\]\s*((?:==|>=)\s*[\d\.]+(?:\s*,\s*<\s*\d+)?)"',
        content,
        re.MULTILINE,
    )
    if version:
        openfisca_core_api = version.group(1)
    version = re.search(r'numpy\s*(>=\s*[\d\.]*,\s*<\d*)"', content, re.MULTILINE)
    if version:
        numpy = version.group(1)
    if not openfisca_core_api or not numpy:
        raise Exception("Dependencies not found in pyproject.toml")
    return {
        "openfisca_tunisia": openfisca_tunisia,
        "openfisca_core_api": openfisca_core_api.replace(" ", ""),
        "numpy": numpy.replace(" ", ""),
    }


def replace_in_file(filepath: str, info: dict):
    """
    ::filepath:: Path to meta.yaml, with filename
    ::info:: Dict with information to populate
    """
    with open(filepath, "rt") as fin:
        meta = fin.read()
    # Replace with info from pyproject.toml
    if PACKAGE_VERSION not in meta:
        raise Exception(f"{PACKAGE_VERSION=} not found in {filepath}")
    meta = meta.replace(PACKAGE_VERSION, info["openfisca_tunisia"])
    if CORE_VERSION not in meta:
        raise Exception(f"{CORE_VERSION=} not found in {filepath}")
    meta = meta.replace(CORE_VERSION, info["openfisca_core_api"])
    if NUMPY_VERSION not in meta:
        raise Exception(f"{NUMPY_VERSION=} not found in {filepath}")
    meta = meta.replace(NUMPY_VERSION, info["numpy"])
    with open(filepath, "wt") as fout:
        fout.write(meta)
    logging.info(
        f"File {filepath} has been updated with informations from pyproject.toml."
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-r",
        "--replace",
        type=bool,
        default=False,
        required=False,
        help="replace in file",
    )
    parser.add_argument(
        "-f",
        "--filename",
        type=str,
        default=".conda/recipe.yaml",
        help="Path to recipe.yaml, with filename",
    )
    parser.add_argument(
        "-o",
        "--only_package_version",
        type=bool,
        default=False,
        help="Only display current package version",
    )
    args = parser.parse_args()
    info = get_versions()
    file = args.filename
    if args.only_package_version:
        print(f'{info["openfisca_tunisia"]}')  # noqa: T201
        exit()
    logging.info("Versions :")
    print(info)  # noqa: T201
    if args.replace:
        logging.info(f"Replace in {file}")
        replace_in_file(file, info)
    else:
        logging.info("Dry mode, no replace made")



"""6. Revenus de valeurs mobilières et de capitaux mobiliers"""

from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class rvcm(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Revenus de valeurs mobilières et de capitaux mobiliers"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        capm_banq = foyer_fiscal.declarant_principal("capm_banq", period=period)
        capm_cent = foyer_fiscal.declarant_principal("capm_cent", period=period)
        capm_caut = foyer_fiscal.declarant_principal("capm_caut", period=period)
        capm_part = foyer_fiscal.declarant_principal("capm_part", period=period)
        capm_oblig = foyer_fiscal.declarant_principal("capm_oblig", period=period)
        capm_caisse = foyer_fiscal.declarant_principal("capm_caisse", period=period)
        capm_plfcc = foyer_fiscal.declarant_principal("capm_plfcc", period=period)
        capm_epinv = foyer_fiscal.declarant_principal("capm_epinv", period=period)
        capm_aut = foyer_fiscal.declarant_principal("capm_aut", period=period)

        return (
            capm_banq
            + capm_cent
            + capm_caut
            + capm_part
            + capm_oblig
            + capm_caisse
            + capm_plfcc
            + capm_epinv
            + capm_aut
        )



import pytest
import datetime

from openfisca_tunisia.scenarios import init_single_entity
from tests.base import tax_benefit_system


scenarios_arguments = [
    dict(
        period=year,
        parent1=dict(
            date_naissance=datetime.date(1972, 1, 1),
            salaire_de_base=2000,
            regime_securite_sociale_cotisant="rsna",
        ),
        parent2=dict(
            date_naissance=datetime.date(1972, 1, 1),
        ),
    )
    for year in range(datetime.date.today().year + 1, 2007, -1)
]


def check_run(simulation, period):
    assert (
        simulation.calculate("revenu_disponible", period=period) is not None
    ), "Can't compute revenu_disponible on period {}".format(period)
    assert (
        simulation.calculate_add("salaire_super_brut", period=period) is not None
    ), "Can't compute salaire_super_brut on period {}".format(period)


@pytest.mark.parametrize("one_scenario_arguments", scenarios_arguments)
def test_basics(one_scenario_arguments):
    scenario = init_single_entity(
        tax_benefit_system.new_scenario(), **one_scenario_arguments
    )
    simulation = scenario.new_simulation(debug=False)
    period = one_scenario_arguments["period"]
    check_run(simulation, period)


@pytest.mark.parametrize("period", range(2007, datetime.date.today().year + 1))
def test_1_parent_axe(period):
    scenario = init_single_entity(
        tax_benefit_system.new_scenario(),
        axes=[
            [
                dict(
                    count=3,
                    name="salaire_imposable",
                    max=100000,
                    min=0,
                )
            ]
        ],
        period=period,
        parent1=dict(date_naissance=datetime.date(period - 40, 1, 1)),
    )
    simulation = scenario.new_simulation()
    assert (
        simulation.calculate("revenu_disponible", period=period) is not None
    ), "Can't compute revenu_disponible on period {}".format(period)
    assert (
        simulation.calculate_add("salaire_super_brut", period=period) is not None
    ), "Can't compute salaire_super_brut on period {}".format(period)



import sys
import os
from io import open

from nbformat import read, write
from nbconvert.preprocessors import ExecutePreprocessor, CellExecutionError

import logging
import traceback


'''
Checks notebooks by trying to convert each one of them into a notebook that
includes the execution results.

This script is similar to running this command:
    jupyter nbconvert --to notebook --execute --ExecutePreprocessor.timeout=60 \
                      --output executed_notebook.ipynotebook demo.ipynb

on a group of notebooks. Whenever an error occurs, this script will give the user
pretty printed output in order to fix it.
'''


log = logging.getLogger(__name__)
log.addHandler(logging.StreamHandler())
log.setLevel(logging.DEBUG)


def is_notebook(file_path):
    return file_path[-6:] == '.ipynb'


def run(notebook_path):
    '''
    Execute a notebook.

    If an error occurs, then save results in a new notebook
    of the name, prefixed by `executed_`.
    '''
    notebook_directory = os.path.dirname(notebook_path)
    notebook_filename = os.path.basename(notebook_path)
    notebook_filename_out = os.path.join(
        notebook_directory,
        'executed_' + notebook_filename
        )

    with open(notebook_path) as f:
        notebook = read(f, as_version = 4)

    try:
        # Execute all the cells in the notebook
        ep = ExecutePreprocessor(timeout = 600, kernel_name = 'python')
        executed_notebook = ep.preprocess(
            notebook,
            {'metadata': {'path': notebook_directory}}
            )

    except CellExecutionError:
        executed_notebook = None
        msg = f'Error executing the notebook {notebook_filename}.\n\n'
        msg += f'See notebook {notebook_filename_out} for stack traceback.'
        log.error(msg)
        raise

    finally:
        with open(notebook_filename_out, mode = 'wt') as f:
            write(notebook, f)
        if executed_notebook is not None:
            os.remove(notebook_filename_out)


# Check script target (file or directory) and test all notebooks
if __name__ == '__main__':
    try:
        target = sys.argv[1]

        if os.path.isdir(target):
            for file in os.listdir(target):
                if is_notebook(file):
                    log.debug('> ' + file)
                    run(os.path.join(target, file))
        else:
            if not is_notebook(target):
                raise Exception(f'Expected an .ipynb file. Got: {target}')
            run(target)

        log.info('OK. No error detected in tested notebook(s).')
    except BaseException as e:
        if len(sys.argv) == 1:
            log.error('Missing notebook or directory containing notebooks to test.')
            log.warn('USAGE: python test_notebooks.py target')
            log.warn('where `target` is directory containing notebooks, or a .ipynb notebook file.')

        else:
            log.debug(str(e.__class__.__name__) + ': ')
            if e.__class__ is CellExecutionError and e.from_cell_and_msg:
                log.debug(e.from_cell_and_msg)
            elif e.message:
                log.debug(e.message)
            log.error(traceback.format_exc())



from openfisca_tunisia.variables.base import *


class allocation_familiale_non_contributive(Variable):
    value_type = float
    entity = Menage
    label = "Allocation familiale pour les ménages non affiliés à un régime de sécurité sociale"
    definition_period = MONTH

    def formula(menage, period, parameters):
        return menage("allocation_familiale_non_contributive_0_5", period) + menage(
            "allocation_familiale_non_contributive_6_18", period
        )


class allocation_familiale_non_contributive_0_5(Variable):
    value_type = float
    entity = Menage
    label = "Allocation familiale pour les enfants entre 0  et 5 ans pour les ménages non affiliés à un régime de sécurité sociale"
    definition_period = MONTH

    def formula_2020_06(menage, period, parameters):
        age = menage.members("age", period)
        eligible = menage("eligible_allocation_familiale_non_contributive_0_5", period)
        enfant_a_charge_0_5 = (age >= 0) * (age <= 5)
        enfants_eligibles = eligible * menage.sum(
            enfant_a_charge_0_5, role=Menage.ENFANT
        )
        allocation_familiale = parameters(
            period
        ).prestations.non_contributives.allocation_familiale
        return enfants_eligibles * allocation_familiale


class allocation_familiale_non_contributive_6_18(Variable):
    value_type = float
    entity = Menage
    label = "Allocation familiale pour les enfants entre 6 et 18 ans pour les ménages non affiliés à un régime de sécurité sociale"
    definition_period = MONTH

    def formula_2022_07(menage, period, parameters):
        age = menage.members("age", period)
        eligible = menage("eligible_allocation_familiale_non_contributive_6_18", period)
        enfant_a_charge_6_18 = (6 <= age) * (age >= 18)
        enfants_eligibles = eligible * menage.sum(
            enfant_a_charge_6_18, role=Menage.ENFANT
        )
        allocation_familiale = parameters(
            period
        ).prestations.non_contributives.allocation_familiale
        return enfants_eligibles * allocation_familiale


class eligible_allocation_familiale_non_contributive_0_5(Variable):
    value_type = bool
    entity = Menage
    label = "Eligibilité à l'allocation familiale non contributive pour les enfants de 0 à 5 ans"
    definition_period = MONTH

    def formula_2020_06(menage, period, parameters):
        # TODO quand est-ce que cela débute ?
        # https://www.unicef.org/mena/media/24061/file
        # Mai 2020: décret abolissant
        # la limite du nombre d’enfants
        # éligibles par ménage (aupar-
        # avant limité à 3 enfants par
        # ménage) et supprimant le
        # seuil d’âge (l’âge minimum
        # pour
        # bénéficier d’une allocation
        # familiale était de plus de 5
        # ans

        return (
            menage("pnafn_eligible", period)
            + menage("amg_1", period)
            + menage("amg_2", period)
            + menage("amen_social_eligible", period)
        )


class eligible_allocation_familiale_non_contributive_6_18(Variable):
    value_type = bool
    entity = Menage
    label = "Eligibilité à l'allocation familiale non contributive pour les enfants de 6 à 18 ans"
    definition_period = MONTH

    def formula_2023_02(menage, period, parameters):
        return (
            menage("pnafn_eligible", period)
            + menage("amg_1", period)
            + menage("amg_2", period)
        )

    def formula_2022_07(menage, period, parameters):
        return menage("pnafn_eligible", period) + menage("amg_1", period)
